@@ -1,8 +1,8 @@
 import tensorflow as tf
 from einops import rearrange
-from fusionlab.functional import tf_iou_score
+from fusionlab.functional.tfiou import tf_iou_score
 
-__all__ = ["TFDiceLoss", "TFDiceCE"]
+__all__ = ["TFIoULoss"]
 
 BINARY_MODE = "binary"
 MULTICLASS_MODE = "multiclass"
